from uuid import UUID

from starlette import status
import fastapi
from tortoise.exceptions import ValidationError

from ..base_ctrl import BaseController
from exceptions import common as common_exc, http as http_exc
from .services import FileService


class FileController(BaseController):
    file_service = FileService()

    async def get_list(self, **kwargs):
        return await self.file_service.get_file_list(**kwargs)

    async def get(self, id: UUID):
        try:
            return await self.file_service.get_file(id)

        except common_exc.NotFoundException as e:
            raise http_exc.HTTPNotFoundException(detail=str(e))

    async def create(self, **kwargs):
        try:
            return await self.file_service.create_file(**kwargs)

        except common_exc.CreateException as e:
            raise http_exc.HTTPBadRequestException(detail=str(e))

        except ValidationError as e:
            raise http_exc.HTTPBadRequestException(detail=f"Validation error: {str(e)}")

    async def update(self, id: UUID, **kwargs):
        try:
            return await self.file_service.update_file(id, **kwargs)

        except common_exc.UpdateException as e:
            raise http_exc.HTTPBadRequestException(detail=str(e))

    async def delete(self, id: UUID):
        try:
            await self.file_service.delete_file(id)
            return fastapi.responses.Response(status_code=status.HTTP_204_NO_CONTENT)

        except common_exc.DeleteException as e:
            raise http_exc.HTTPBadRequestException(detail=str(e))

        except common_exc.NotFoundException as e:
            raise http_exc.HTTPNotFoundException(detail=str(e))
