from uuid import UUID

from starlette import status
import fastapi
from tortoise.exceptions import ValidationError

from ..base_ctrl import BaseController
from exceptions import common as common_exc, http as http_exc
from .services import FolderService


class FolderController(BaseController):
    folder_service = FolderService()

    async def get_list(self, **kwargs):
        return await self.folder_service.get_folder_list(**kwargs)

    async def get(self, id: UUID):
        try:
            return await self.folder_service.get_folder(id)

        except common_exc.NotFoundException as e:
            raise http_exc.HTTPNotFoundException(detail=str(e))

    async def create(self, **kwargs):
        try:
            return await self.folder_service.create_folder(**kwargs)

        except common_exc.CreateException as e:
            raise http_exc.HTTPBadRequestException(detail=str(e))

        except ValidationError as e:
            raise http_exc.HTTPBadRequestException(detail=f"Validation error: {str(e)}")

    async def update(self, id: UUID, **kwargs):
        try:
            return await self.folder_service.update_folder(id, **kwargs)

        except common_exc.UpdateException as e:
            raise http_exc.HTTPBadRequestException(detail=str(e))

    async def delete(self, id: UUID):
        try:
            await self.folder_service.delete_folder(id)
            return fastapi.responses.Response(status_code=status.HTTP_204_NO_CONTENT)

        except common_exc.DeleteException as e:
            raise http_exc.HTTPBadRequestException(detail=str(e))

        except common_exc.NotFoundException as e:
            raise http_exc.HTTPNotFoundException(detail=str(e))
